#include "stdafx.h"
#include <opencv2\opencv.hpp>
#include <iostream>
#include <iterator>
#include <fstream>
#include "BezierHelper.h"
using namespace std;
using namespace cv; 

//Read segmentation result. The result file lists all points 
//on the boundary. For example:
//x1 y1
//x2 y2 ...
//xn yn
vector<Point> read_output(string filename)
{
	vector<Point> pts;
	ifstream ifs(filename.c_str());
	float x, y;
	while (!ifs.eof())
	{
		ifs >> x >> y;
		//The output file can contain floating points, but
		//in our evaluation we must use discrete integers. 
		pts.push_back(Point((int)x, (int)y));
	}
	ifs.close();

	//Squeeze the list - remove repeating points.
	pts.erase(unique(pts.begin(), pts.end(), [](Point a, Point b) {return a.x == b.x && a.y == b.y; }), pts.end());
	return pts;
}

//Read groundtruth file. The format is described in readme.txt.
vector<Point> read_gt(string filename, bool squeeze)
{ 
	vector<Point> pts;
	vector<Point2f> v;
	vector<Point2f> t1, t2;
	ifstream ifs(filename.c_str());
	string tmp;
	ifs >> tmp;
	int num;
	ifs >> num;
	ifs >> tmp;
	float x, y;
	for (int i = 0; i < num; i++)
	{
		ifs >> x >> y; 
		v.push_back(Point2f(x, y));
	}
	ifs >> tmp;
	for (int i = 0; i < num; i++)
	{
		ifs >> x >> y; 
		t1.push_back(Point2f(x, y));
	}
	ifs >> tmp;
	for (int i = 0; i < num; i++)
	{
		ifs >> x >> y; 
		t2.push_back(Point2f(x, y));
	}
	ifs.close();

	BezierCurve bc;
	for (int i = 0; i < num; i++)
	{
		//Each Bezier curve is 100px maximum.
		vector<Point2f> seg = bc.Bezier2D(v[i], t1[i], t2[i], v[(i + 1) % num]);
		for (auto it = seg.begin(); it != seg.end(); it++)
			pts.push_back(Point((int)(*it).x, (int)(*it).y));
	}
	
	if(squeeze)
		//Squeeze the list - remove repeating points.
		//We don't do so for evaluating temporal consistency,
		//otherwise the correspondency will be broken.
		pts.erase(unique(pts.begin(), pts.end(), [](Point a, Point b) {return a.x == b.x && a.y == b.y; }), pts.end());
	return pts;
}

//Compute spatial accuracy
float acc(vector<Point> gt, vector<Point> res, float th)
{
	int hit = 0;	
	for (int i = 0; i < gt.size(); i++)
	{
		if(abs(pointPolygonTest(res, gt[i], true)) <= th) 
			hit++;
	}
	return (float)hit / gt.size();
}

//Compute temporal consistency
float cons(vector<Point> gt0, vector<Point> gt1, vector<Point> res0, vector<Point> res1, float th)
{
	assert(gt0.size() == gt1.size());
	vector<float> dist0, dist1;
	int hit = 0;
	for (int i = 0; i < gt0.size(); i++) 
		dist0.push_back(pointPolygonTest(res0, gt0[i], true));
	for (int i = 0; i < gt1.size(); i++)
		dist1.push_back(pointPolygonTest(res1, gt1[i], true));
	for (int i = 0; i < gt0.size(); i++)
	{
		if (abs(dist0[i] - dist1[i]) <= th) 
			hit++;
	}
	return (float)hit / gt0.size();
}

int main()
{
	//parameter th please refer to the paper
	float th = 1;

	//Example 1: measuring spatial accuracy
	auto gt_accu_0 = read_gt("gt0.txt", true);
	auto gt_accu_1 = read_gt("gt1.txt", true);
	auto res_0 = read_output("res0.txt");
	auto res_1 = read_output("res1.txt");
	cout << "Example 1: accuracy 0 = " << acc(gt_accu_0, res_0, th) << endl;
	cout << "Example 1: accuracy 1 = " << acc(gt_accu_1, res_1, th) << endl;
	 
	//Example 2: measuring temporal consistency 
	auto gt_cons_0 = read_gt("gt0.txt", false);
	auto gt_cons_1 = read_gt("gt1.txt", false);
	cout << "Example 2: consistency = " << cons(gt_cons_0, gt_cons_1, res_0, res_1, th) << endl; 

	return 0;
}

